package cn.skcks.study.rxjava;

import io.reactivex.rxjava3.core.Flowable;
import io.reactivex.rxjava3.core.Observable;
import io.reactivex.rxjava3.disposables.Disposable;
import io.reactivex.rxjava3.schedulers.Schedulers;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;

import java.util.concurrent.TimeUnit;

@Slf4j
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.NONE)
public class RxJavaStudyStarterTest {
    @Test
    void contextLoads() {
      log.info("Hello, RxJava!");
        Disposable disposable = Flowable.just("Hello world").subscribe(System.out::println);
        disposable.dispose();

        // Observable<Long> longObservable = Observable.create((ObservableOnSubscribe<Long>) emitter -> {
        //     for (long i = 1L; i <= 5L; i++) {
        //         emitter.onNext(i);
        //         Thread.sleep((long) (Math.random() * 100));
        //     }
        //     emitter.onComplete();
        // }).subscribeOn(Schedulers.io())
        //         .observeOn(Schedulers.newThread())
        //         .doOnError(e -> log.error("Error: ", e));

        Observable<Long> intervalObservable = Observable.interval(20, TimeUnit.MILLISECONDS)
                .take(50)
                .subscribeOn(Schedulers.io())
                .observeOn(Schedulers.newThread())
                .doOnError(e -> log.error("Error: ", e));

        Disposable delayDisposable = intervalObservable.subscribe(item -> {
            try {
                log.debug("println: Subscribe Next: {}", item);
                Thread.sleep(100);
            } catch (InterruptedException e) {
                log.error("执行中断 ", e);
            }

        }, e -> {
            log.error("Error: ", e);
        }, () -> log.info("Complete"));

        intervalObservable.subscribe(item -> log.info("[logInfo] Subscribe Next: {}", item));
        new Thread(()->{
            try {
                Thread.sleep(200);
            } catch (InterruptedException ignored) {

            }
            delayDisposable.dispose();
        }).start();
        intervalObservable.blockingSubscribe();

        Observable<Integer> take = Observable.fromArray(1, 2, 3, 4, 5)
                .observeOn(Schedulers.computation()).skip(2).take(1);
        take.subscribe(System.out::println);
    }
}
